SystemOrganization addCategory: #'XML-Parser-HTML-Tests'!

TestCase subclass: #XMLHTMLParserTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-HTML-Tests'!

!XMLHTMLParserTest methodsFor: 'source' stamp: 'monty 11/16/2014 21:02'!
addressBookXML
	^ XMLParserTest addressBookXML! !

!XMLHTMLParserTest methodsFor: 'source' stamp: 'monty 11/16/2014 21:02'!
addressBookXMLWithDTD
	^ XMLParserTest addressBookXMLWithDTD! !

!XMLHTMLParserTest methodsFor: 'benchmarking' stamp: 'monty 11/7/2014 19:35'!
benchmark1
	"
	self new benchmark1
	"
	Smalltalk garbageCollect.
	^ [4000 timesRepeat: [SAXHTMLHandler parse: XMLParserTest addressBookXML]] timeToRun! !

!XMLHTMLParserTest methodsFor: 'benchmarking' stamp: 'monty 11/7/2014 19:35'!
benchmark2
	"
	self new benchmark2
	"
	Smalltalk garbageCollect.
	^ [4000 timesRepeat: [XMLHTMLParser parse: XMLParserTest addressBookXML]] timeToRun! !

!XMLHTMLParserTest methodsFor: 'tests' stamp: 'monty 1/5/2015 06:52'!
testCharacterReferences
	self assert:
		(XMLHTMLParser parse: '&#60; &#x3C &#X3E &#62; &#0; &#-1 &#invalid; &#invalid &#')
			contentString = '< < > > &#0; &#-1 &#invalid; &#invalid &#'.! !

!XMLHTMLParserTest methodsFor: 'tests' stamp: 'monty 1/5/2015 06:54'!
testCharacterReferencesUnreplaced
	self assert:
		((XMLHTMLParser on: '&#60; &#x3C &#X3E &#62; &#0; &#-1 &#invalid; &#invalid &#')
			replacesContentEntityReferences: false;
			parseDocument) contentString = '&#60; &#x3C &#X3E &#62; &#0; &#-1 &#invalid; &#invalid &#'! !

!XMLHTMLParserTest methodsFor: 'tests' stamp: 'monty 1/5/2015 05:58'!
testHTMLEntities
	| entity character document p |

	entity := 'nbsp'.
	character := XMLHTMLParserTokenizer predefinedEntities at: entity.
	document :=
		XMLHTMLParser parse:
			'<p>&', entity, ';&', entity, '</p>',
			'<div>&undeclared; &undeclared &;& ;'.

	p := document findElementNamed: 'p'.
	self
		assert: p contentString size = 2;
		assert: (p contentString allSatisfy: [:each | each == character]);
		assert: (document findElementNamed: 'div') contentString = '&undeclared; &undeclared &;& ;'.! !

!XMLHTMLParserTest methodsFor: 'tests' stamp: 'monty 1/5/2015 06:52'!
testHTMLEntitiesUnreplaced
	| entity character document |

	entity := 'nbsp'.
	character := XMLHTMLParserTokenizer predefinedEntities at: entity.
	document :=
		(XMLHTMLParser on:
			'<p>&', entity, ';&', entity, '</p>',
			'<div>&undeclared; &undeclared &;& ;')
			replacesContentEntityReferences: false;
			parseDocument.
	self
		assert: (document findElementNamed: 'p') contentString = '&nbsp;&nbsp';
		assert: (document findElementNamed: 'div') contentString = '&undeclared; &undeclared &;& ;'.! !

!XMLHTMLParserTest methodsFor: 'tests' stamp: 'monty 12/12/2014 08:37'!
testIgnoringInternalSubsets
	| html docWithSubset docWithoutSubset |

	html :=
		'<html>
			<head><title>test</title></head>
			<body><p>test</p></body>
		</html>'.
	docWithSubset := XMLHTMLParser parse: '<!!DOCTYPE html [
		<!!ELEMENT html ANY>
		<!!ENTITY lang "en-us">
		<!!ATTLIST html xml:lang CDATA #FIXED "&lang;">
		<!!--end-->]>', html.
	self
		deny: docWithSubset doctypeDeclaration isNil;
		assert: docWithSubset doctypeDeclaration root = 'html';
		deny: docWithSubset doctypeDeclaration hasChildren.

	docWithoutSubset := XMLHTMLParser parse: '<!!DOCTYPE html>', html.
	self
		deny: docWithoutSubset doctypeDeclaration isNil;
		assert: docWithoutSubset doctypeDeclaration root = 'html';
		deny: docWithoutSubset doctypeDeclaration hasChildren.

	self assert: docWithSubset printString = docWithoutSubset printString.! !

!XMLHTMLParserTest methodsFor: 'tests' stamp: 'monty 5/24/2015 23:59'!
testInnerXML
	| document addressBook |

	document := XMLHTMLParser parse: self addressBookXML.
	addressBook := document findElementNamed: 'addressbook'.
	addressBook firstNode innerXML: 'first'.
	self assert: addressBook firstNode innerXML = 'first'.
	addressBook nodes second innerXML: '<second>second</second>'.
	self assert: addressBook nodes second innerXML = '<second>second</second>'.
	addressBook nodes third innerXML: '<!!--third-->'.
	self assert: addressBook nodes third innerXML = ''.
	addressBook lastNode innerXML: '<!![CDATA[not ignored]]>'.
	self assert: addressBook lastNode innerXML = 'not ignored'.
	self
		assert: addressBook nodes size = 4;
		assert: (addressBook nodes allSatisfy: [:each | each isElementNamed: 'person']).! !

!XMLHTMLParserTest methodsFor: 'tests' stamp: 'monty 11/16/2014 21:04'!
testInnerXMLElement
	| document addressBook person personStartTag |

	document := XMLHTMLParser parse: self addressBookXML.
	addressBook := document findElementNamed: 'addressbook'.
	person := addressBook firstNode.
	personStartTag := person printString copyUpTo: $>.
	addressBook removeNode: person.
	person innerXML: '<p>new</p>'.
	self
		assert: person innerXML = '<p>new</p>';
		assert: person printString = (personStartTag, '><p>new</p></person>').! !

!XMLHTMLParserTest methodsFor: 'tests' stamp: 'monty 3/24/2015 21:01'!
testIsInContent
	| html handler parser |

	html := '<?xml version="1.0"?>
			<!!--comment-->
			<?target data?>
			<!!DOCTYPE root [<!!ELEMENT root EMPTY>]>
			<!!--comment-->
			<?target data?>
			<root/>'.
	handler := SAXHTMLHandler on: html.
	parser := handler parser.
	9 timesRepeat: [
		parser parseToken.
		self deny: handler isInContent].
	parser parseToken.
	self assert: handler isInContent.

	handler := SAXHTMLHandler on: 'test'.
	parser := handler parser.
	parser parseToken.
	self deny: handler isInContent.
	parser parseToken.
	self assert: handler isInContent.! !

!XMLHTMLParserTest methodsFor: 'tests' stamp: 'monty 4/7/2015 00:37'!
testParseHTML
	| html |

	html := '<html><body>test</body></html>'.
	self
		assert: html parseHTML printString = html;
		assert: html readStream parseHTML printString = html.! !

!XMLHTMLParserTest methodsFor: 'tests' stamp: 'monty 11/7/2014 19:34'!
testParsingAttributes
	| document element |

	document := XMLHTMLParser parse: '<test one="quoted" two=''subQuoted'' three=unquoted four'.
	element := document findElementNamed: 'test'.
	self
		assert: (element attributeAt: 'one') = 'quoted';
		assert: (element attributeAt: 'two') = 'subQuoted';
		assert: (element attributeAt: 'three') = 'unquoted';
		assert: (element attributeAt: 'four') isEmpty.! !

!XMLHTMLParserTest methodsFor: 'tests' stamp: 'monty 5/25/2015 00:06'!
testParsingCDATA
	#(
		'<body><!![CDATA[not ignored]]></body>'
		'<!!DOCTYPE><!![CDATA[not ignored]]><html><body/>')
		do: [:each | | document |
			document := XMLHTMLParser parse: each.
			self assert: document contentString = 'not ignored'].! !

!XMLHTMLParserTest methodsFor: 'tests' stamp: 'monty 3/22/2015 17:23'!
testParsingComments
	| document |

	document :=
		(XMLHTMLParser on: '<body><!!--comment can have -- before end --></body>')
			preservesCommentNodes: true;
			parseDocument.
	self
		assert: (document findElementNamed: 'body') firstNode isComment;
		assert: (document findElementNamed: 'body') firstNode string = 'comment can have -- before end '! !

!XMLHTMLParserTest methodsFor: 'tests' stamp: 'monty 4/5/2015 03:06'!
testParsingDTDs
	| xml document |

	xml :=
		'<!!DOCTYPE root PUBLIC "pubid""external.dtd" [
			<!!ELEMENT root ( #PCDATA | one | two)* >
			<!!ATTLIST root
						%entity;
						name CDATA #IMPLIED
						number ( %one; | two | three ) "three">
			<!!ENTITY general "replacement">
			<!!ENTITY externalGeneral PUBLIC "pubid2" "external1.dtd">
			<!!ENTITY % parameter "<!!ENTITY generalTwo &#32replacement&#32>">
			%parameter;
			<!!ENTITY % externalParameter SYSTEM "external2.dtd">
			<!!NOTATION notation SYSTEM "note.doc">]>
		<p>&generalTwo;</p>'.
	self
		shouldnt: [document := XMLHTMLParser parse: xml]
		raise: XMLParserException.
	self
		assert: document nodes size = 1;
		assert: (document findElementNamed: 'p') contentString = '&generalTwo;'.
	self assert:
		document printString =
			(XMLHTMLParser parse:
				'<!!DOCTYPE root PUBLIC "pubid" "external.dtd"><p>&generalTwo;</p>') printString.! !

!XMLHTMLParserTest methodsFor: 'tests' stamp: 'monty 11/7/2014 19:34'!
testParsingForms
	| document |

	document := XMLHTMLParser parse: '<form><input><form><input>'.

	self assert:
		(document allElements asArray collect: [:each | each name]) =
			#('html' 'body' 'form' 'input' 'form' 'input').

	document allElementsNamed: 'form' do: [:each |
		self
			assert: each elements size = 1;
			assert: (each firstElement isNamed: 'input')].
	document allElementsNamed: 'input' do: [:each |
		self deny: each hasChildren].! !

!XMLHTMLParserTest methodsFor: 'tests' stamp: 'monty 11/7/2014 19:34'!
testParsingImplicitStartTag
	| paragraphs |

	paragraphs := (XMLHTMLParser parse: '</p></p>') allElementsNamed: 'p'.
	self
		assert: paragraphs size = 2;
		deny: (paragraphs anySatisfy: [:each | each hasChildren]).! !

!XMLHTMLParserTest methodsFor: 'tests' stamp: 'monty 11/7/2014 19:34'!
testParsingInvalidPCData
	| document |

	#('< one & two >' '& three < four >') do: [:each |
		document := XMLHTMLParser parse: each.
		self
			assert: document allNodes size = 4;
			assert: (document findElementNamed: 'body') contentString = each].! !

!XMLHTMLParserTest methodsFor: 'tests' stamp: 'monty 11/7/2014 19:34'!
testParsingNonHTMLSections
	| document |

	document := XMLHTMLParser parse:
		'<head><Title><b>"title</b></TITLE><SCRIPT>',
		'1 < 2; 3 > 2; document.write("</Script>");</Script>
		<Style>//"</STYLE><HEAD>'.
	self
		assert: (document findElementNamed: 'title') contentString = '<b>"title</b>';
		assert: (document findElementNamed: 'script') contentString =
			'1 < 2; 3 > 2; document.write("</Script>");';
		assert: (document findElementNamed: 'style') contentString = '//"'.! !

!XMLHTMLParserTest methodsFor: 'tests' stamp: 'monty 11/7/2014 19:34'!
testParsingTables
	| document tables trs tds |

	document := XMLHTMLParser parse: '<table><tr><td>one<td>two<tr><td>three<table><tr><td>four'.
	self assert:
		(document allElements asArray collect: [:each | each name]) =
			#('html' 'body' 'table' 'tr' 'td' 'td' 'tr' 'td' 'table' 'tr' 'td').

	tables := document allElementsNamed: 'table'.
	self
		assert: tables first elements size = 2;
		assert: tables second elements size = 1.
	tables do: [:each | self assert: (each elements allSatisfy: [:element | element isNamed: 'tr'])].

	trs := document allElementsNamed: 'tr'.
	self
		assert: trs first elements size = 2;
		assert: trs second elements size = 1;
		assert: trs third elements size = 1.
	trs do: [:each | self assert: (each elements allSatisfy: [:element | element isNamed: 'td'])].

	tds := document allElementsNamed: 'td'.
	self
		assert: tds first contentString = 'one';
		assert: tds second contentString = 'two';
		assert: tds third contentString = 'three four';
		assert: (tds third nodes second isNamed: 'table').! !

!XMLHTMLParserTest methodsFor: 'tests' stamp: 'monty 11/7/2014 19:34'!
testParsingText
	| document |
	
	document := XMLHTMLParser parse: 'text'.
	self
		assert: document allNodes size = 4;
		assert: (document allElementsCollect: [:each | each name]) asArray = #('html' 'body');
		assert: (document findElementNamed: 'body') contentString = 'text'.! !

!XMLHTMLParserTest methodsFor: 'tests' stamp: 'monty 11/7/2014 19:34'!
testParsingVoidElements
	| document |

	document := XMLHTMLParser parse: '<body>one<img>two</img><body>three'.
	self
		assert: (document allElementsCollect: [:each | each name]) asArray = #('html' 'body' 'img');
		assert: (document findElementNamed: 'body') contentString = 'one twothree';
		deny: (document findElementNamed: 'img') hasChildren.! !

!XMLHTMLParserTest methodsFor: 'tests' stamp: 'monty 11/16/2014 21:04'!
testParsingXML
	self
		shouldnt: [
			SAXHTMLHandler
				parse: self addressBookXML;
				parse: self addressBookXMLWithDTD;
				parse: XMLDOMParserTest addressBookNamespacedXML;
				parse: XMLDOMParserTest addressBookNamespacePrefixedXML]
		raise: XMLParserException! !

!XMLHTMLParserTest methodsFor: 'tests' stamp: 'monty 11/7/2014 19:34'!
testProcessingInstructions
	| document |

	document := XMLHTMLParser parse: '<body><?ignored data?></body>'.
	self deny: (document findElementNamed: 'body') hasChildren.! !

!XMLHTMLParserTest methodsFor: 'tests' stamp: 'monty 11/7/2014 19:34'!
testReopeningHTML
	| document |

	document := XMLHTMLParser parse: '<html><B></body></html><I>'.
	self
		assert: (document allElementsCollect: [:each | each name]) asArray = #('html' 'body' 'b' 'i');
		assert: document root nodes size = 1;
		assert: (document root firstElement elementsCollect: [:each | each name]) asArray = #('b' 'i').

	document := XMLHTMLParser parse: '<html><B>bold</body></html>normal'.
	self
		assert: (document allElementsCollect: [:each | each name]) asArray = #('html' 'body' 'b');
		assert: (document findElementNamed: 'body') contentString = 'bold normal'.! !

!XMLHTMLParserTest methodsFor: 'tests' stamp: 'monty 11/7/2014 19:34'!
testReopeningHeadSection
	| document |

	document := XMLHTMLParser parse: '<head><p>text<Title>reopened'.
	self
		assert: (document root elementsCollect: [:each | each name]) asArray = #('head' 'body');
		assert: (document findElementNamed: 'head') elements size = 1;
		assert: ((document findElementNamed: 'head') elementAt: 'title') contentString = 'reopened';
		assert: (document findElementNamed: 'body') elements size = 1;
		assert: ((document findElementNamed: 'body') elementAt: 'p') contentString = 'text'.! !

!XMLHTMLParserTest methodsFor: 'tests' stamp: 'monty 4/5/2015 08:03'!
testXMLDeclaration
	| document |

	document := XMLHTMLParser parse: '<?xml ?>'.
	self
		assert: document version = 1.0;
		assert: document encoding isEmpty;
		deny: document isStandalone.

	document := XMLHTMLParser parse: '<?xml version="onePointZero" encoding="utf-8" standalone="yes"?>'.
	self
		assert: document version = 1.0;
		assert: document encoding = 'utf-8';
		deny: document isStandalone.! !
